import * as moment from 'moment';
import { forwardRef, Inject, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Modulo } from '../modulo/entity/modulo.entity';
import { ModuloMotivo } from './entity/modulo-motivo.entity';
import { Operador } from '../operador/entity/operador.entity';
import { ModuloService } from '../modulo/modulo.service';

@Injectable()
export class ModuloMotivoService {
  constructor(
    @InjectRepository(ModuloMotivo)
    private repository: Repository<ModuloMotivo>,
    @Inject(forwardRef(() => ModuloService))
    private moduloService: ModuloService,
  ) {}

  create(
    operador: Operador,
    modulo: Modulo,
    motivo: string,
    numero_alumnos: number,
    fecha_creacion: string,
  ) {
    return this.repository.save(
      this.repository.create({
        motivo,
        numero_alumnos,
        operador,
        modulo,
        fecha_creacion: moment(fecha_creacion).toDate(),
      }),
    );
  }

  findAllByIdModulo(id_modulo: number, pagina: number) {
    return this.moduloService.findById(id_modulo).then((modulo) =>
      this.repository.findAndCount({
        where: { modulo },
        skip: (pagina - 1) * 25,
        take: 25,
      }),
    );
  }
}
